#include <iostream>
#include <algorithm>
#include <vector>
#include <set>
#include <map>
#include <cassert>

using namespace std;

#define int long long

const int INF = 1e10 + 239;
const int N = 3e5 + 7;

struct Edge {
	int u, v, c;

	Edge() {}

	Edge(int u_, int v_, int c_) {
		u = u_;
		v = v_;
		c = c_;
	}

	bool operator<(const Edge &other) const {
		return c < other.c;
	}
};

namespace DSU {
	int p[N];
	int rank[N];

	int get(int v) {
		if (p[v] != v) {
			p[v] = get(p[v]);
		}
		return p[v];
	} 

	int merge(int a, int b) {
		a = get(a);
		b = get(b);
		if (a == b) {
			return 0;
		} else {
			if (rank[a] < rank[b]) swap(a, b);
			p[b] = a;
			if (rank[a] == rank[b]) rank[a]++;
			return 1;
		}
	}

	void init(int n) {
		for (int i = 0; i < n; i++) {
			p[i] = i;
			rank[i] = 0;
		}
	}
}

vector<Edge> E;

int ans[N];
vector<pair<int, int>> g[N];

void dfs(int cur, int p, int mn, int mx) {
	ans[cur] = min(ans[cur], mx + mn);
	for (auto [t, w] : g[cur]) {
		if (t != p) {
			dfs(t, cur, min(mn, w), max(mx, w));
		}
	}
}

signed main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);

	int n, m;
	cin >> n >> m;
	for (int i = 0; i < m; i++) {
		int u, v, c;
		cin >> u >> v >> c;
		u--;
		v--;
		E.push_back(Edge(u, v, c)); 
	}
	sort(E.begin(), E.end());
	DSU::init(n);
	for (auto e : E) {
		if (DSU::merge(e.u,  e.v)) {
			g[e.u].push_back({e.v, e.c});
			g[e.v].push_back({e.u, e.c});
		}
	}
	fill(ans, ans + n, INF);
	dfs(0, -1, INF, -INF);
	for (int i = 1; i < n; i++) {
		cout << ans[i] << '\n';
	}
}