#include <iostream>
#include <algorithm>
#include <numeric>
#include <limits>
#include <cassert>

template<class T> bool ckmin(T& a, T b) { return b < a ? a = b, true : false; }
template<class T> bool ckmax(T& a, T b) { return a < b ? a = b, true : false; }

using namespace std;
using pi = pair<int, int>;

const int N = 3e5+7;

int n, q, h[N], p[N], med[N], ans[N], tin[N], tout[N], curva[N];
int gt;
vector<int> vertx[N];
vector<pi> g[N];

struct Edge {
    int a, b, w;

    bool operator < (const Edge& e) {
        return w < e.w;
    }
};

struct Dsu {
    int n;
    vector<int> p, r;

    Dsu(int n) : n(n), p(n), r(n) {
        iota(p.begin(), p.end(), 0);
    }

    void clear() {
        iota(p.begin(), p.end(), 0);
        fill(r.begin(), r.end(), 0);
    }

    int root(int v) {
        return v == p[v] ? v : p[v] = root(p[v]);
    }

    bool unite(int a, int b) {
        a = root(a), b = root(b);
        if (a != b) {
            if (r[a] < r[b]) swap(a, b);
            p[b] = a;
            r[a] += r[a] == r[b];
            return true;
        } else {
            return false;
        }
    }
};

struct CkminTree {
    int n;
    vector<int> t;

    CkminTree(int n) : n(n), t(2*n, numeric_limits<int>::max()) { }

    void update(int l, int r, int x) {
        assert(x >= 0);
        for (l += n, r += n; l <= r; l >>= 1, r >>= 1) {
            if ((l&1) == 1) ckmin(t[l++], x);
            if ((r&1) == 0) ckmin(t[r--], x);
        }
    }

    void _clear(int i) {
        if (i != 0) {
            _clear(i >> 1);
            assert(t[i] >= 0);
            if (i < n) ckmin(t[2*i], t[i]), ckmin(t[2*i+1], t[i]);
            t[i] = numeric_limits<int>::max();
        }
    }

    void clear(int i, int x) { 
        assert(x >= 0);
        _clear(i+n); 
        t[i+n] = x;
    }

    int get(int i) {
        int res = numeric_limits<int>::max();
        for (i += n; i != 0; i >>= 1) res = min(res, t[i]);
        return res;
    }
};

void dfs(int v, int path = numeric_limits<int>::max(), int dp = -1) {
    tin[v] = gt++;
    med[v] = path;
    p[v] = dp;
    for (auto [to, w] : g[v]) {
        if (to == dp) continue;
        h[to] = h[v] + 1;
        p[to] = v;
        dfs(to, min(path, w), v);
    }
    tout[v] = gt;
}

bool cmp(int i, int j) { return h[i] < h[j]; }

void unite_strong_group(int a, int b, Dsu& dsu) {
    a = dsu.root(a), b = dsu.root(b);
    int v = min(curva[a], curva[b], cmp);
    int m = min(med[a], med[b]);
    dsu.unite(a, b);
    curva[a] = curva[b] = v;
    med[a] = med[b] = m;
}

int up(int v, Dsu& dsu) {
    return p[curva[dsu.root(v)]];
}

int lca(int a, int b, Dsu& dsu) {
    while (true) {
        a = curva[dsu.root(a)];
        if (tin[a] <= tin[b] && tout[b] <= tout[a]) break;
        a = p[a];
    }
    return a;
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0);
    int n, m; cin >> n >> m;
    // if (n > 5000) return 0;
    vector<Edge> e(m);
    for (auto& [a, b, w] : e) {
        cin >> a >> b >> w;
        a--;
        b--;
    }
    sort(e.begin(), e.end());
    Dsu dsu(n), dsu2(n);

    for (auto& [a, b, w] : e) {
        if (dsu.unite(a, b)) {
            g[a].emplace_back(b, w);
            g[b].emplace_back(a, w);
        }
    }

    iota(curva,curva+n,0);

    fill(ans, ans+n, numeric_limits<int>::max());
    dfs(0);

    CkminTree tree(n), minw(n);
    for (int i = 0; i < n; ++i) minw.t[tin[i]+n] = med[i];
    for (int i = 0; i < n; ++i) vertx[i].push_back(i);

    dsu.clear();
    for (auto& [a, b, w] : e) {
        bool need_to_upd = true;
        int pp = dsu.root(a), qq = dsu.root(b);
        if (!dsu.unite(a, b)) {
            if (dsu2.root(a) != dsu2.root(b)) {
                int g = lca(a, b, dsu2);
                for (int u = a; dsu2.root(u) != dsu2.root(g);) {
                    int t = up(u, dsu2);
                    unite_strong_group(u, g, dsu2);
                    u = t;
                }
                for (int u = b; dsu2.root(u) != dsu2.root(g);) {
                    int t = up(u, dsu2);
                    unite_strong_group(u, g, dsu2);
                    u = t;
                }

                int gg = curva[dsu2.root(g)];
                tree.update(tin[gg], tout[gg]-1, w+med[dsu2.root(g)]);
                minw.update(tin[gg], tout[gg]-1, med[dsu2.root(g)]);
            } else {
                need_to_upd = false;
            }
        } else {
            int rr = dsu.root(pp);
            if (pp != rr) for (int i : vertx[pp]) vertx[rr].push_back(i);
            if (qq != rr) for (int i : vertx[qq]) vertx[rr].push_back(i);

            for (int i : vertx[dsu.root(0)]) {
                if (i) {
                    tree.clear(tin[i], minw.get(tin[i]) + w);
                }
            }
            vertx[dsu.root(0)].clear();
        }
    }

    for (int i = 1; i < n; ++i) cout << tree.get(tin[i]) << '\n';
}