#include <iostream>
#include <algorithm>
#include <numeric>

using namespace std;
using pi = pair<int, int>;

const int N = 3e5+7;
const int INF = 1e9;

int n, q, h[N], p[N], med[N], used[N], ans[N], ucnt;
vector<pi> g[N];

struct Edge {
    int a, b, w;

    bool operator < (const Edge& e) {
        return w < e.w;
    }
};

struct Dsu {
    int n;
    vector<int> p, r;

    Dsu(int n) : n(n), p(n), r(n) {
        iota(p.begin(), p.end(), 0);
    }

    void clear() {
        iota(p.begin(), p.end(), 0);
        fill(r.begin(), r.end(), 0);
    }

    int root(int v) {
        return v == p[v] ? v : p[v] = root(p[v]);
    }

    bool unite(int a, int b) {
        a = root(a), b = root(b);
        if (a != b) {
            if (r[a] < r[b]) swap(a, b);
            p[b] = a;
            r[a] += r[a] == r[b];
            return true;
        } else {
            return false;
        }
    }
};

void dfs(int v, int path = INF, int dp = -1) {
    med[v] = path;
    p[v] = dp;
    for (auto [to, w] : g[v]) {
        if (to == dp) continue;
        h[to] = h[v] + 1;
        p[to] = v;
        dfs(to, min(path, w), v);
    }
}

int lca(int a, int b) {
    ++ucnt; // clear used
    while (true) {
        if (a != -1) {
            if (used[a] == ucnt) return a;
            used[a] = ucnt;
            a = p[a];
        }

        if (b != -1) {
            if (used[b] == ucnt) return b;
            used[b] = ucnt;
            b = p[b];
        }
    }
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0);
    int n, m; cin >> n >> m;
    vector<Edge> e(m);
    for (auto& [a, b, w] : e) {
        cin >> a >> b >> w;
        a--;
        b--;
    }
    sort(e.begin(), e.end());
    Dsu dsu(n), dsu2(n);

    for (auto& [a, b, w] : e) {
        if (dsu.unite(a, b)) {
            g[a].emplace_back(b, w);
            g[b].emplace_back(a, w);
        }
    }

    fill(ans, ans+n, 2*INF);
    dfs(0);
    dsu.clear();
    for (auto& [a, b, w] : e) {
        if (!dsu.unite(a, b)) {
            if (dsu2.root(a) != dsu2.root(b)) {
                int g = lca(a, b), smn = med[dsu2.root(g)];
                for (int u = a; u != g; u = p[u]) dsu2.unite(u, g), smn = min(smn, med[dsu2.root(u)]);
                for (int u = b; u != g; u = p[u]) dsu2.unite(u, g), smn = min(smn, med[dsu2.root(u)]);
                med[dsu2.root(g)] = smn;

                for (int i = 1; i < n; ++i) {
                    if (dsu.root(i) == dsu.root(0)) {
                        int m = med[dsu2.root(i)];
                        for (int u = i; u >= 0; u = p[u]) m = min(m, med[dsu2.root(u)]);
                        ans[i] = min(ans[i], w + m);
                    }
                }
            }
        } else {
            for (int i = 1; i < n; ++i) {
                if (dsu.root(i) == dsu.root(0)) {
                    int m = med[dsu2.root(i)];
                    for (int u = i; u >= 0; u = p[u]) m = min(m, med[dsu2.root(u)]);
                    ans[i] = min(ans[i], w + m);
                }
            }
        }
    }

    for (int i = 1; i < n; ++i) cout << ans[i] << '\n';
}