#include <iostream>
#include <algorithm>
#include <numeric>
#include <limits>

using namespace std;
using pi = pair<int, int>;

const int N = 3e5+7;

int n, q, h[N], p[N], med[N], used[N], ans[N], tin[N], tout[N], curva[N], ucnt;
int gt;
vector<pi> g[N];

struct Edge {
    int a, b, w;

    bool operator < (const Edge& e) {
        return w < e.w;
    }
};

struct Dsu {
    int n;
    vector<int> p, r;

    Dsu(int n) : n(n), p(n), r(n) {
        iota(p.begin(), p.end(), 0);
    }

    void clear() {
        iota(p.begin(), p.end(), 0);
        fill(r.begin(), r.end(), 0);
    }

    int root(int v) {
        return v == p[v] ? v : p[v] = root(p[v]);
    }

    bool unite(int a, int b) {
        a = root(a), b = root(b);
        if (a != b) {
            if (r[a] < r[b]) swap(a, b);
            p[b] = a;
            r[a] += r[a] == r[b];
            return true;
        } else {
            return false;
        }
    }
};

void dfs(int v, int path = numeric_limits<int>::max(), int dp = -1) {
    tin[v] = gt++;
    med[v] = path;
    p[v] = dp;
    for (auto [to, w] : g[v]) {
        if (to == dp) continue;
        h[to] = h[v] + 1;
        p[to] = v;
        dfs(to, min(path, w), v);
    }
    tout[v] = gt;
}

bool cmp(int i, int j) { return h[i] < h[j]; }

void unite_strong_group(int a, int b, Dsu& dsu) {
    a = dsu.root(a), b = dsu.root(b);
    int v = min(curva[a], curva[b], cmp);
    int m = min(med[a], med[b]);
    dsu.unite(a, b);
    curva[a] = curva[b] = v;
    med[a] = med[b] = m;

}

int up(int v, Dsu& dsu) {
    return p[curva[dsu.root(v)]];
}

int lca(int a, int b, Dsu& dsu) {
    while (true) {
        a = curva[dsu.root(a)];
        if (tin[a] <= tin[b] && tout[b] <= tout[a]) break;
        a = p[a];
    }
    return a;
}

int main() {
    ios::sync_with_stdio(0); cin.tie(0);
    int n, m; cin >> n >> m;
    vector<Edge> e(m);
    for (auto& [a, b, w] : e) {
        cin >> a >> b >> w;
        a--;
        b--;
    }
    sort(e.begin(), e.end());
    Dsu dsu(n), dsu2(n);

    for (auto& [a, b, w] : e) {
        if (dsu.unite(a, b)) {
            g[a].emplace_back(b, w);
            g[b].emplace_back(a, w);
        }
    }

    fill(ans, ans+n, numeric_limits<int>::max());
    dfs(0);
    dsu.clear();
    for (auto& [a, b, w] : e) {
        if (!dsu.unite(a, b)) {
            if (dsu2.root(a) != dsu2.root(b)) {
                int g = lca(a, b, dsu2);
                for (int u = a; dsu.root(u) != dsu.root(g); u = up(u, dsu)) unite_strong_group(u, g, dsu2);
                for (int u = b; dsu.root(u) != dsu.root(g); u = up(u, dsu)) unite_strong_group(u, g, dsu2);

                for (int i = 1; i < n; ++i) {
                    if (dsu.root(i) == dsu.root(0)) {
                        int m = med[dsu2.root(i)];
                        for (int u = i; u >= 0; u = p[u]) m = min(m, med[dsu2.root(u)]);
                        ans[i] = min(ans[i], w + m);
                    }
                }
            }
        } else {
            for (int i = 1; i < n; ++i) {
                if (dsu.root(i) == dsu.root(0)) {
                    int m = med[dsu2.root(i)];
                    for (int u = i; u >= 0; u = p[u]) m = min(m, med[dsu2.root(u)]);
                    ans[i] = min(ans[i], w + m);
                }
            }
        }
    }

    for (int i = 1; i < n; ++i) cout << ans[i] << '\n';
}