#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

const ll INF = 1e9;
const ll INFLL = 1e18;

struct Edge{
    ll fr, to, tm, mark;
};

void dfs2(int v, int p, vector<int> &vis2, vector<ll> &ans, vector<vector<pair<ll, ll> > > &g)
{
    //cout << v << " " << p << endl;
    vis2[v] = 1;
    for (auto e : g[v]) if (e.first != p)
    {
        int u = e.first;
        ans[u] = min(ans[u], min(ans[v], e.second));
        dfs2(u, v, vis2, ans, g);
    }
}

vector<ll> solve_fixed_mx(int n, int m, vector<int> color,
                          vector<Edge> bridges, vector<Edge> bibridges,
                          int lim)
{
    //cout << "Start !" << endl;
    //for (int i = 0; i < n; i++) cout << color[i] << " ";
    //cout << endl;

    int cl = *max_element(color.begin(), color.end()) + 1;
    vector<vector<pair<ll, ll> > > comp_g(cl);
    vector<ll> ans(cl, INFLL);
    for (auto e : bridges)
    {
        int v = e.fr, u = e.to;
        if (color[v] == color[u]) ans[color[v]] = min(ans[color[v]], e.tm);
        else
        {
            //cout << "! add " << color[v] << " " << color[u] << " (" << v << ",  " << u << ") " << endl;
            comp_g[color[v]].push_back({color[u], e.tm});
            comp_g[color[u]].push_back({color[v], e.tm});
        }
    }
    for (auto e : bibridges)
    {
        int v = e.fr, u = e.to;
        if (color[v] == color[u]) ans[color[v]] = min(ans[color[v]], e.tm);
    }

    //cout << "*ok" << endl;

    vector<int> vis2(cl);
    dfs2(color[0], color[0], vis2, ans, comp_g);

    //cout << "*2" << endl;

    vector<ll> res(n);
    for (int i = 1; i < n; i++)
    {
        res[i] = ans[color[i]] + lim;
        //cout << res[i] << " ";
    }
    //cout << endl;
    //cout << "Fin" << endl;
    return res;
}

int Root(int x, vector<int> &root)
{
    if (x == root[x]) return x;
    return root[x] = Root(root[x], root);
}

void Merge(int v, int u, vector<int> &root, vector<int> &sz)
{
    v = Root(v, root);
    u = Root(u, root);
    if (sz[v] < sz[u])
    {
        sz[u] += sz[v];
        root[v] = u;
    }
    else
    {
        sz[v] += sz[u];
        root[u] = v;
    }
}

void dfs(int v, int p, vector<int> &par, vector<vector<int> > &g)
{
    par[v] = p;
    for (auto u : g[v]) if (u != p) dfs(u, v, par, g);
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    int n, m;
    cin >> n >> m;
    vector<int> fr(m), to(m), tm(m);
    for (int i = 0; i < m; i++)
    {
        cin >> fr[i] >> to[i] >> tm[i];
        fr[i]--, to[i]--;
    }
    vector<int> order(m);
    iota(order.begin(), order.end(), 0);
    sort(order.begin(), order.end(), [&](int i, int j){
         return tm[i] < tm[j];
         });

    vector<int> root(n), sz(n, 1);
    iota(root.begin(), root.end(), 0); // connected components

    vector<int> biroot(n), bisz(n, 1);
    iota(biroot.begin(), biroot.end(), 0); // biconnected components

    vector<ll> ans(n, INFLL);
    int calls = 0;

    vector<Edge> bridges, bibridges;
    vector<vector<int> > spawner(n);

    for (int i = 0; i < m; i++)
    {
        //cout << "Start " << i << endl;
        int v = fr[order[i]], u = to[order[i]], Tm = tm[order[i]];
        if (Root(v, root) != Root(u, root))
        {
            Merge(v, u, root, sz);
            bridges.push_back({v, u, Tm, i});
            spawner[v].push_back(u);
            spawner[u].push_back(v);
            vector<int> color(n);
            for (int i = 0; i < n; i++) color[i] = Root(i, biroot);
            vector<ll> res = solve_fixed_mx(n, m, color, bridges, bibridges, Tm);
            for (int i = 0; i < n; i++) if (Root(i, root) == Root(0, root)) ans[i] = min(ans[i], res[i]);
            calls++;
        }
        else if (Root(v, biroot) != Root(u, biroot))
        {
            Merge(v, u, biroot, sz);
            bibridges.push_back({v, u, Tm, i});

            vector<int> p(n);
            dfs(v, v, p, spawner);

            while (u != v)
            {
                Merge(v, u, biroot, sz);
                u = p[u];
            }

            vector<int> color(n);
            for (int i = 0; i < n; i++) color[i] = Root(i, biroot);
            vector<ll> res = solve_fixed_mx(n, m, color, bridges, bibridges, Tm);
            for (int i = 0; i < n; i++) if (Root(i, root) == Root(0, root)) ans[i] = min(ans[i], res[i]);
            calls++;
        }
        //cout << "End " << i << endl;
    }
    assert(calls <= 2 * n);
    for (int i = 1; i < n; i++)
        cout << ans[i] << "\n";
}



/*
6 7
1 2 1000000000
2 3 2
3 4 3
4 1 1000000000
1 5 1000000000
5 6 1
5 6 2

4 4
1 2 2
3 2 0
2 4 3
4 3 1

7 10
1 2 2
1 2 8
2 3 3
3 4 5
3 5 4
4 5 4
6 5 7
6 4 4
1 7 6
6 7 9
*/
