#include <iostream>
#include <vector>
#include <cmath>
#include <unordered_set>
#include <set>

#define nei_set1(v) (is_parent ? nodes[v].children : nodes[v].parents)
#define nei_set2(v) (!is_parent ? nodes[v].children : nodes[v].parents)

using namespace std;

struct Node {
    int idx;

    set<int> children;
    set<int> parents;
};

struct Gadget {
    int v1, v2, v3;
};

bool is_leaf(const Node &node) {
    if (node.children.size() + node.parents.size() == 1) return true;

    if (node.children.size() == 1 && node.parents.size() == 1
        && *node.children.begin() == *node.parents.begin())
        return true;
    return false;
}

bool is_crossing(const Node &node) {
    if (node.children.size() > 2 || node.parents.size() > 2) return true;
    if (node.children.size() + node.parents.size() <= 2) return false;

    if (node.children.size() == 2) {
        return node.children.count(*node.parents.begin()) != 0;
    } else {
        return node.parents.count(*node.children.begin()) != 0;
    }
}

bool operator<(const Node &n1, const Node& n2) {
   bool c1 = is_crossing(n1), c2 = is_crossing(n2);

    if (c1 == c2) return n1.idx < n2.idx;
    else return c1 < c2;
}

int main() {
    std::ios::sync_with_stdio(false);

    int n, m;
    cin >> n >> m;

    vector<Node> nodes(n);
    unordered_set<int> leaves;
    for (int i = 0; i < n; ++i) nodes[i].idx = i;

    for (int i = 0; i < m; ++i) {
        int a, b;
        cin >> a >> b;

        a--;
        b--;

        nodes[a].children.insert(b);
        nodes[b].parents.insert(a);
    }

    for (auto &node : nodes) {
        if (is_leaf(node)) leaves.insert(node.idx);
    }

    vector<Gadget> res;

    while (!leaves.empty()) {
        int v1 = -1, v2 = -1;
        for (int v : leaves) {
            bool is_parent = nodes[v].children.size() == 1;

            int vv = *nei_set1(v).begin();

            if (is_crossing(nodes[vv])) continue;

            v1 = v;
            v2 = vv;
            break;
        }
        if (v1 == -1) {
            v1 = *leaves.begin();
            bool is_parent = nodes[v1].children.size() == 1;

            v2 = *nei_set1(v1).begin();
        }
        bool is_parent = nodes[v1].children.size() == 1;

        if (nei_set1(v2).empty()) {
            cout << "No" << endl;
            return 0;
        } else {
            int v3;
            if (nei_set1(v2).count(v1) != 0) v3 = v1;
            else v3 = *nei_set1(v2).begin();

            nei_set2(v3).erase(v2);
            nei_set1(v2).erase(v3);

            nei_set2(v2).erase(v1);
            nei_set1(v1).erase(v2);

            if (is_parent) res.push_back({v1 + 1, v2 + 1, v3 + 1});
            else res.push_back({v3 + 1, v2 + 1, v1 + 1});

            if (is_leaf(nodes[v1])) leaves.insert(v1);
            else leaves.erase(v1);

            if (is_leaf(nodes[v2])) leaves.insert(v2);
            else leaves.erase(v2);

            if (is_leaf(nodes[v3])) leaves.insert(v3);
            else leaves.erase(v3);
        }
    }

    if (res.size() == m / 2) {
        cout << "Yes\n";

        for (auto gad : res) {
            cout << gad.v1 << " " << gad.v2 << " " << gad.v2 << " " << gad.v3 << "\n";
        }

        cout.flush();
    } else cout << "No" << endl;
}
