﻿#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

const int INF = 1e9;

struct e1 {
	int u, v;
	bool rev;

	e1() = default;
	e1(int u1, int v1) {
		if (u1 < v1) {
			u = u1;
			v = v1;
			rev = false;
		}
		else {
			rev = true;
			u = v1;
			v = u1;
		}
	}
	bool operator<(const e1& other) {
		return make_pair(u, v) < make_pair(other.u, other.v);
	}
	bool operator==(const e1& other) {
		return make_pair(u, v) == make_pair(other.u, other.v);
	}
};

struct edge {
	int to;
	bool out, in;

	edge() = default;
	edge(int to, bool out, bool in) : to(to), out(out), in(in) {}
};

edge dfs(int v, vector<int>& used, vector<vector<edge>>& g, vector<pair<int, int>>& ans) {
	used[v] = 1;
	vector<int> in, out;
	edge x;
	for (edge& i : g[v]) {
		if (used[i.to]) {
			continue;
		}
		x = dfs(i.to, used, g, ans);
		if (x.to == INF) {
			return x;
		} else if (x.out == 1) {
			if (i.out) {
				ans.emplace_back(v, i.to);
				ans.emplace_back(i.to, x.to);
				i.out = false;
			}
			else {
				return edge(INF, 0, 0);
			}
		} else if (x.in) {
			if (i.in) {
				ans.emplace_back(x.to, i.to);
				ans.emplace_back(i.to, v);
				i.in = false;
			}
			else {
				return edge(INF, 0, 0);
			}
		}
		if (i.in) {
			in.push_back(i.to);
		}
		if (i.out) {
			out.push_back(i.to);
		}
	}
	while (!in.empty() && !out.empty()) {
		ans.emplace_back(in.back(), v);
		ans.emplace_back(v, out.back());
		in.pop_back();
		out.pop_back();
	}
	if (in.empty()) {
		if (out.size() > 1) {
			return edge(INF, 0, 0);
		}
		if (out.size() == 1) {
			return edge(out.back(), 1, 0);
		}
		return edge(-1, 0, 0);
	}
	else {
		if (in.size() > 1) {
			return edge(INF, 0, 0);
		}
		if (in.size() == 1) {
			return edge(in.back(), 0, 1);
		}
		return edge(-1, 0, 0);
	}
}

int main() {
	int n, m, u, v;
	cin >> n >> m;
	vector<e1> edges1(m);
	for (int i = 0; i < m; ++i) {
		cin >> u >> v;
		--u;
		--v;
		edges1[i] = e1(u, v);
	}
	vector<vector<edge>> g(n);
	sort(edges1.begin(), edges1.end());
	for (int i = 0; i < m; ++i) {
		u = edges1[i].u;
		v = edges1[i].v;
		if (i + 1 < m && (edges1[i].v == edges1[i + 1].v && edges1[i].u == edges1[i + 1].u)) {
			g[u].push_back(edge(v, 1, 1));
			g[v].push_back(edge(u, 1, 1));
			++i;
		}
		else if (edges1[i].rev) {
			g[v].push_back(edge(u, 1, 0));
			g[u].push_back(edge(v, 0, 1));
		}
		else {
			g[u].push_back(edge(v, 1, 0));
			g[v].push_back(edge(u, 0, 1));
		}
	}

	vector<int> used(n, 0);
	vector<pair<int, int>> ans;
	edge x;
	for (int i = 0; i < n; ++i) {
		if (!used[i]) {
			x = dfs(i, used, g, ans);
			if (x.to != -1) {
				cout << "No";
				return 0;
			}
		}
	}
	cout << "Yes\n";
	for (int i = 0; i < m; i += 2) {
		cout << ans[i].first + 1 << ' ' << ans[i].second + 1 << ' ' << ans[i + 1].first + 1 << ' ' << ans[i + 1].second + 1 << '\n';
	}

	return 0;
}
