#include <iostream>
#include <vector>
#include <set>
#include <algorithm>

void dfs(int32_t vertex, int32_t parent, std::set<int32_t>* graph, std::vector<std::vector<int32_t> >& ans, bool* visited) {
    visited[vertex] = true;
    for(auto it = graph[vertex].begin(); it != graph[vertex].end(); it++) {
        if(*it == parent) {
            ans.push_back({parent, vertex, parent});
        } else {
            dfs(*it, vertex, graph, ans, visited);
        }
    }
    graph[vertex].clear();
}

std::vector<int32_t> search(int32_t vertex, std::set<int32_t>* graph, int32_t* indegree, int32_t* outdegree, int32_t invoc) {
    if(invoc == 2)
        return {outdegree[vertex] < indegree[vertex] ? vertex : -1};
    else if(invoc == 1) {
        for(auto it = graph[vertex].begin(); it != graph[vertex].end(); it++) {
            std::vector<int32_t> res2 = search(*it, graph, indegree, outdegree, invoc + 1);
            if(res2[0] != -1)
                return {vertex, res2[0]};
        }
        return {-1, -1};
    } else {
        for(auto it = graph[vertex].begin(); it != graph[vertex].end(); it++) {
            std::vector<int32_t> res1 = search(*it, graph, indegree, outdegree, invoc + 1);
            if(res1[0] != -1)
                return {vertex, res1[0], res1[1]};
        }
        return {};
    }
}

std::vector<int32_t> mild_search(int32_t vertex, std::set<int32_t>* graph, int32_t invoc) {
    if(invoc == 2)
        return {vertex};
    else if(invoc == 1) {
        for(auto it = graph[vertex].begin(); it != graph[vertex].end(); it++) {
            std::vector<int32_t> res2 = mild_search(*it, graph, invoc + 1);
            if(res2[0] != -1)
                return {vertex, res2[0]};
        }
        return {-1, -1};
    } else {
        for(auto it = graph[vertex].begin(); it != graph[vertex].end(); it++) {
            std::vector<int32_t> res1 = mild_search(*it, graph, invoc + 1);
            if(res1[0] != -1)
                return {vertex, res1[0], res1[1]};
        }
        return {};
    }
}

int main() {
    int32_t n, m;
    std::cin >> n >> m;

    std::set<int32_t>* graph = new std::set<int32_t>[n];
    std::set<int32_t>* rev_graph = new std::set<int32_t>[n];
    int32_t* indeg = new int32_t[n], *outdeg = new int32_t[n];
    for(int32_t i = 0; i < n; i++) {
        indeg[i] = 0;
        outdeg[i] = 0;
    }
    for(int32_t i = 0; i < m; i++) {
        int32_t src, dst;
        std::cin >> src >> dst;
        src--;
        dst--;

        graph[src].insert(dst);
        rev_graph[src].insert(dst);
        indeg[dst]++;
        outdeg[src]++;
    }

    /*int32_t* arr = new int32_t[n];
    for(int32_t i = 0; i < n; i++)
        arr[i] = i;
    while(std::next_permutation(arr, arr + n)) {
        for(int32_t i = 0; i < n; i++)
            if()
    }*/

    std::vector<std::vector<int32_t> > ans;
    for(int32_t i = 0; i < m / 2; i++) {
        std::vector<int32_t> new_gadget;

        //a cycle of size 2
        for(int32_t j = 0; j < n; j++)
            if(indeg[j] == 1 && outdeg[j] == 1) {
                new_gadget = {*rev_graph[j].begin(), j, *graph[j].begin()};
            }

        //other stuff
        if(new_gadget.size() == 0) {
            for (int32_t j = 0; j < n; j++)
                if (outdeg[j] > indeg[j]) {
                    std::vector<int32_t> cur = search(j, graph, indeg, outdeg, 0);
                    if (cur.size() > 0)
                        new_gadget = cur;
                }
        }

        if(new_gadget.size() == 0) {
            break;
        }

        outdeg[new_gadget[0]]--;
        indeg[new_gadget[1]]--;
        outdeg[new_gadget[1]]--;
        indeg[new_gadget[2]]--;

        graph[new_gadget[0]].erase(new_gadget[1]);
        graph[new_gadget[1]].erase(new_gadget[2]);
        rev_graph[new_gadget[2]].erase(new_gadget[1]);
        rev_graph[new_gadget[1]].erase(new_gadget[0]);

        ans.push_back(new_gadget);
    }

    //greedily match the rest
    /*bool* visited = new bool[n];
    for(int32_t i = 0; i < n; i++)
        visited[i] = false;
    for(int32_t i = 0; i < n; i++) {
        if(!visited[i])
            dfs(i, -1, graph, ans, visited);
    }*/

    if(ans.size() != m / 2)
        std::cout << "No";
    else {
        std::cout << "Yes" << "\n";
        for(int32_t i = 0; i < m / 2; i++)
            std::cout << ans[i][0] + 1 << " " << ans[i][1] + 1 << " " << ans[i][1] + 1 << " " << ans[i][2] + 1 << "\n";
    }
    return 0;
}