#include <iostream>
#include <vector>
#include <sstream>
#include <set>
#include <map>
#include <assert.h>
#include <cmath>
#include <algorithm>

using namespace std;
typedef long long LL;
typedef unsigned long long ULL;

struct Edge {
	int from, to, id, cost;
	bool operator < (const Edge& a) const {
		return tie(from, to, id, cost) < tie(a.from, a.to, a.id, a.cost);
	}
};

struct Graph {
	int size;
	set<Edge> edges;
	vector<Edge> edgesVec;
	vector<bool> used;
	vector<int> minOnPath;
	vector<int> maxOnPath;

	Graph(){}

	Graph(int s) {
		size = s;
		//neighbours.resize(s);
		minOnPath.resize(s);
		maxOnPath.resize(s);
		used.resize(s, false);
	}
	
	void addEdge(int a, int b, int cost) {
		//neighbours[a].push_back(b);
		//neighbours[b].push_back(a);
		//costs[{a, b}] = cost;
		//costs[{b, a}] = cost;
		Edge f = Edge({ a, b, (int)edgesVec.size(), cost }), s = Edge({ b, a, (int)edgesVec.size(), cost });
		edges.insert(f);
		edges.insert(s);

		edgesVec.push_back(f);
		//edgesVec.push_back(s);
	}

	void dfs(int v, int edgeId = -1) {
		used[v] = true;

		if (edgeId != -1) {
			Edge lastEdge = edgesVec[edgeId];
			minOnPath[v] = edgesVec[edgeId].cost;
			maxOnPath[v] = edgesVec[edgeId].cost;

			int parent = lastEdge.from == v ? lastEdge.to : lastEdge.from;

			minOnPath[v] = min(edgesVec[edgeId].cost, minOnPath[parent]);
			maxOnPath[v] = max(edgesVec[edgeId].cost, maxOnPath[parent]);
		}
		else {
			minOnPath[v] = INT_MAX;
			maxOnPath[v] = 0;
		}
		
		for (auto edgePtr = edges.lower_bound(Edge({ v, 0, 0, 0 })); edgePtr != edges.end() && edgePtr->from == v; edgePtr++) {
			int u = edgePtr->to;
			if (!used[u]) {
				dfs(u, edgePtr->id);
			}
		}

		for (auto edgePtr = edges.lower_bound(Edge({ v, 0, 0, 0 })); edgePtr != edges.end() && edgePtr->from == v; edgePtr++) {
			int u = edgePtr->to;
			if (used[u] && edgePtr->id != edgeId) {
				minOnPath[u] = min(minOnPath[v], minOnPath[u]);
				maxOnPath[u] = max(maxOnPath[u], maxOnPath[v]);
			}
		}
	}
};

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	int n, m;
	cin >> n >> m;

	Graph G(n);

	for (int i = 0; i < m; i++)
	{
		int a, b, cost;
		cin >> a >> b >> cost;
		a--;
		b--;
		G.addEdge(a, b, cost);
	}

	G.dfs(0);

	for (int i = 1; i < n; i++)
	{
		cout << G.maxOnPath[i] + G.minOnPath[i] << endl;
	}

}