#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <cassert>
#include <random>

using namespace std;

mt19937 rng(10);

const int C = 30;
const int MOD = 1e9 + 7;
const int BASE = 7; 

int add(int a, int b) {
	if (a + b < MOD) {
		return a + b;
	}
	return a + b - MOD;
}

int sub(int a, int b) {
	if (a >= b) {
		return a - b;
	}
	return a - b + MOD;
}

int mul(int a, int b) {
	return 1LL * a * b % MOD;
}

string encode(string s) {
	string ret;
	int n = (int)s.size();
	ret = s;
	int h = 0;
	for (auto t : s) {
		h = mul(h, BASE);
		h = add(h, t - '0');
	}
	int c = 0;
	for (int i = 0; i < C; i++) {
		c += ((h >> i) & 1);
		ret.push_back('0' + ((h >> i) & 1));
	}
	ret.push_back('0' + (c & 1));
	return ret;
}

string decode(string s) {
	int n = (int)s.size() - C - 1;
	{
		int c = 0;
		for (int i = n; i < n + C; i++) {
			c += (s[i] - '0');
		}
		if ((c & 1) != s[n + C] - '0') {
			return s.substr(0, n);
		}
	}
	int h = 0;
	for (int i = n; i < n + C; i++) {
		h += (s[i] - '0') * (1 << (i - n));
	}
	int ch = 0;
	for (int i = 0; i < n; i++) {
		ch = mul(ch, BASE);
		ch = add(ch, s[i] - '0');
	}
	string ans = s.substr(0, n);
	if (h == ch) {
		return ans;
		exit(0);
	}
	int cp = 1;
	for (int i = n - 1; i >= 0; i--) {
		ch = sub(ch, mul(cp, s[i] - '0'));
		int nch = add(ch, mul(cp, 1 - (s[i] - '0')));
		if (nch == h) {
			ans[i] = '0' + '1' - ans[i];
			return ans;
		}
		ch = add(ch, mul(cp, s[i] - '0'));
		cp = mul(cp, BASE);
	}
	assert(false);
}

void run() {
	int n = 1e5;
	string s(n, '$');
	for (int i = 0; i < n; i++) {
		s[i] = '0' + (rng() & 1);
	}
	string code = encode(s);
	int i = rng() % (int)code.size();
	code[i] = '0' + '1' - code[i];
	string res = decode(code);
	assert(res == s);
}

signed main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);

	// int t;
	// cin >> t;
	// string s;
	// cin >> s;
	// if (t == 1) {
	// 	s = encode(s);
	// } else {
	// 	s = decode(s);
	// }
	// cout << s << endl;

	while (true) {
		run();
	}
}